SystemOrganization addCategory: #'Zippers-Support'!
SystemOrganization addCategory: #Zippers!
SystemOrganization addCategory: #'Zippers-BST'!
SystemOrganization addCategory: #'Zippers-Tests'!

Object subclass: #BinarySearchTree
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-BST'!

!BinarySearchTree commentStamp: 'fbs 2/5/2011 23:10' prior: 0!
I implement a binary search tree in a strictly functional manner.!

Object subclass: #ListZipper
	instanceVariableNames: 'focus trail'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!ListZipper commentStamp: 'fbs 12/28/2010 16:27' prior: 0!
A zipper is a structure that allows one, in a purely functional manner, to navigate a structure. It works by remembering the steps taken to focus on any particular part of a structure.

A ListZipper is a zipper for any sequenceable collection. It has two instance variables, namely

focus - the (sub)list on which we are currently focussed.
trail - the steps taken to focus on the focus instance variable.!

!ListZipper class methodsFor: 'instance creation' stamp: 'fbs 12/28/2010 16:48'!
on: aSequenceableCollection
	^ self new focus: aSequenceableCollection asOrderedCollection trail: OrderedCollection new.! !

!ListZipper methodsFor: 'mutating' stamp: 'fbs 12/30/2010 20:50'!
change: aBlock
	"Given an arity 1 block, replace the current value in the list with that of evaluating aBlock on that value."
	
	^ ListZipper new
		focus: (aBlock value: focus first) asOrderedCollection, focus allButFirst
		trail: trail.! !

!ListZipper methodsFor: 'accessing' stamp: 'fbs 12/28/2010 16:27'!
focus
	^ focus.! !

!ListZipper methodsFor: 'private' stamp: 'fbs 12/28/2010 16:36'!
focus: aSeqColl trail: anotherSeqColl
	focus := aSeqColl.
	trail := anotherSeqColl.! !

!ListZipper methodsFor: 'navigating' stamp: 'fbs 12/30/2010 23:22'!
left
	trail isEmpty ifTrue: [^ ZipperNavigation new signal: 'Left at the first element'].
	
	^ ListZipper new
		focus: (focus copy addFirst: trail first; yourself)
		trail: trail allButFirst.! !

!ListZipper methodsFor: 'navigating' stamp: 'fbs 12/30/2010 23:22'!
right
	focus isEmpty ifTrue: [^ ZipperNavigation new signal: 'Right at the last element'].
	
	^ ListZipper new
		focus: focus allButFirst
		trail: (trail copy addFirst: focus first; yourself)! !

Object subclass: #PreOrderTraversal
	instanceVariableNames: 'root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!PreOrderTraversal commentStamp: 'fbs 2/13/2011 17:04' prior: 0!
I provide a pre-order traversal of an n-tree: I visit the value of a node and then its subchildren.!

!PreOrderTraversal methodsFor: 'private' stamp: 'fbs 2/17/2011 07:57'!
depthFirstTraversal: aZipper doing: aBlock
	"A naive depth-first traversal. It fails in the presence of cycles."
	| value |
	"aZipper isNil if we're backtracking."
	aZipper ifNil: [^ nil].
	
	value := aBlock value: aZipper focus value.
	
	^ aZipper isEmpty
		ifTrue: [ value ]
		ifFalse: [ self depthFirstTraversal:
			(aZipper downOrElse:
				[self depthFirstTraversal:
					(aZipper rightOrElse: [
						self depthFirstTraversal:
							(aZipper up rightOrElse: [nil "All nodes processed; up again!!"])
							doing: aBlock])
					doing: aBlock])
			doing: aBlock ]! !

!PreOrderTraversal methodsFor: 'enumerating' stamp: 'fbs 2/14/2011 08:30'!
traverse: aTree doing: aBlock
	self depthFirstTraversal: aTree zipper doing: aBlock.! !

Object subclass: #TreeZipper
	instanceVariableNames: 'focus trail'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!TreeZipper commentStamp: 'fbs 1/1/2011 18:01' prior: 0!
I am a zipper for navigating over a variable-arity tree, that is a tree that may have any number of children per node.

That is, I allow one to navigate arbitrarily over a tree structure (a ZTree), and change parts of it, where "change" means "return a structure that shares part of the original structure, but otherwise leaves the host structure unchanged".!

!TreeZipper class methodsFor: 'instance creation' stamp: 'fbs 12/31/2010 16:36'!
on: aTree
	^ self new focus: aTree trail: nil.! !

!TreeZipper methodsFor: 'mutating' stamp: 'fbs 2/9/2011 09:15'!
change: aBlock
	"aBlock is a unary block whose evaluated value we use to replace the current focus."
	^ self changeTo: (aBlock value: focus value).! !

!TreeZipper methodsFor: 'mutating' stamp: 'fbs 12/31/2010 19:32'!
changeTo: newValue
	^ TreeZipper new
		focus: (ZTree
			value: newValue
			children: focus children)
		trail: trail.! !

!TreeZipper methodsFor: 'copying' stamp: 'fbs 2/16/2011 07:45'!
copy
	^ TreeZipper new
		focus: self focus
		trail: self trail.! !

!TreeZipper methodsFor: 'mutating' stamp: 'fbs 1/1/2011 17:17'!
delete
	trail ifNil: [^ ZipperNavigation signal: 'Delete at top'].
	
	(trail leftNodes isEmpty and: [trail rightNodes isEmpty]) ifTrue:
		[^ TreeZipper new
			focus: (ZTree value: trail value) 
			trail: trail path].
			
	trail leftNodes isEmpty ifTrue:
		[^ TreeZipper new
			focus: trail rightNodes first
			trail: (TreeZipperNode
				path: trail path
				value: trail value
				leftNodes: trail leftNodes
				rightNodes: trail rightNodes allButFirst)].
			
	^ TreeZipper new
		focus: trail leftNodes first
		trail: (TreeZipperNode
			path: trail path
			value: trail value
			leftNodes: trail leftNodes allButFirst
			rightNodes: trail rightNodes).! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:33'!
down
	"Return a Zipper focusing on the first child element of this node."
	^ self move: #safeDown.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:37'!
downOrElse: aBlock
	^ self move: #safeDown orElse: aBlock.! !

!TreeZipper methodsFor: 'accessing' stamp: 'fbs 12/30/2010 22:06'!
focus
	^ focus.! !

!TreeZipper methodsFor: 'private' stamp: 'fbs 12/31/2010 16:37'!
focus: aTree trail: aTreeZipperNode
	focus := aTree.
	trail := aTreeZipperNode.! !

!TreeZipper methodsFor: 'mutating' stamp: 'fbs 12/31/2010 23:46'!
insertDown: anObject
	^ TreeZipper new
		focus: (ZTree value: anObject)
		trail: (TreeZipperNode
			path: trail
			value: focus value
			leftNodes: OrderedCollection new
			rightNodes: focus children).! !

!TreeZipper methodsFor: 'mutating' stamp: 'fbs 1/1/2011 15:49'!
insertLeft: anObject
	trail ifNil: [^ ZipperNavigation signal: 'Insert at top'].
	^ TreeZipper new
		focus: focus
		trail: (TreeZipperNode
			path: trail path
			value: trail value
			leftNodes: (trail leftNodes copy addFirst: (ZTree value: anObject); yourself)
			rightNodes: trail rightNodes)! !

!TreeZipper methodsFor: 'mutating' stamp: 'fbs 1/1/2011 00:10'!
insertRight: anObject
	trail ifNil: [^ ZipperNavigation signal: 'Insert at top'].
	^ TreeZipper new
		focus: focus
		trail: (TreeZipperNode
			path: trail path
			value: trail value
			leftNodes: trail leftNodes
			rightNodes: (trail rightNodes copy addFirst: (ZTree value: anObject); yourself))! !

!TreeZipper methodsFor: 'testing' stamp: 'fbs 2/17/2011 07:43'!
isEmpty
	"We don't have a concept of an empty tree (a tree with no nodes). A tree with a single node is the closest we get. So an empty tree is a tree where we can go neither up nor down."
	| moveDown moveUp |
	moveDown := self safeDown.
	moveDown first = #success ifTrue: [ self safeUp ].
	moveUp := self safeUp.
	moveDown first = #success ifTrue: [ self safeUp ].
	
	^ (moveDown first ~= #success) and: [moveUp first ~= #success]! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:34'!
left
	^ self move: #safeLeft.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:36'!
leftOrElse: aBlock
	^ self move: #safeLeft orElse: aBlock.! !

!TreeZipper methodsFor: 'private' stamp: 'fbs 2/16/2011 17:43'!
move: directionSelector
	"Move in some direction. If we fall off the data structure, raise an exception."
	| move |
	move := self perform: directionSelector.
	move first = #success ifFalse: [ ^ ZipperNavigation signal: move first ].
	
	^ move second.! !

!TreeZipper methodsFor: 'private' stamp: 'fbs 2/16/2011 17:43'!
move: directionSelector orElse: aBlock
	"Move in some direction. If we can't, run aBlock. If aBlock takes parameters, pass in the current focus as an argument."
	| move |
	move := self perform: directionSelector.
	^ move first = #success
		ifTrue: [ move second ]
		ifFalse: [ aBlock cull: self focus ].! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:34'!
right
	^ self move: #safeRight.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:36'!
rightOrElse: aBlock
	^ self move: #safeRight orElse: aBlock.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 12/31/2010 23:29'!
root
	"Return the possibly new/mutated tree under navigation"
	^ trail
		ifNil: [focus]
		ifNotNil: [self up root]! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 07:54'!
safeDown
	"Return a pair whose first element tells us whether successfully went down, and a Zipper. If yes, a Zipper focusing on the first child element of this node. If not, a Zipper focused on where we are."
	| success zipper |
	success := self focus children isEmpty
		ifTrue: [ #downAtLeafNode ]
		ifFalse: [ #success ].
	
	zipper := (success = #success) ifTrue:
		[TreeZipper new
			focus: focus children first
			trail: (TreeZipperNode
					path: trail
					value: focus value
					leftNodes: OrderedCollection new
					rightNodes: focus children allButFirst)
		] ifFalse: [self copy].
	^ {success. zipper.}! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 07:58'!
safeLeft
	| success zipper |
	trail
		ifNil: [ success := #leftAtRootNode ]
		ifNotNil: [trail leftNodes ifEmpty: [ success := #leftAtLeftmostSibling ]].
	success ifNil: [ success := #success ].
	
	zipper := (success = #success) ifTrue:
		[TreeZipper new
			focus: trail leftNodes first
			trail: (TreeZipperNode
				path: trail path
				value: trail value
				leftNodes: (trail leftNodes allButFirst)
				rightNodes: (trail rightNodes addFirst: focus; yourself))]
		ifFalse: [ self copy ].
	^ {success. zipper.}! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:27'!
safeRight
	| success zipper |
	trail
		ifNil: [ success := #rightAtRootNode ]
		ifNotNil: [ trail rightNodes ifEmpty: [ success := #rightAtRightmostSibling ]].
	success ifNil: [ success := #success ].
	
	zipper := (success = #success)
		ifTrue: [ TreeZipper new
			focus: trail rightNodes first
			trail: (TreeZipperNode
				path: trail path
				value: trail value
				leftNodes: (trail leftNodes addFirst: focus; yourself)
				rightNodes: trail rightNodes allButFirst) ]
		ifFalse: [ self copy ].
		
	^ {success. zipper.}! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:31'!
safeUp
	| success zipper |
	success := trail
		ifNil: [ #upAtRootNode ]
		ifNotNil: [ #success ].
	
	zipper := (success = #success)
		ifTrue: [ TreeZipper new
			focus: (ZTree
				value: trail value
				children: trail leftNodes reverse, focus asOrderedCollection, trail rightNodes)
			trail: trail path ]
		ifFalse: [ self copy ].
		
	^ {success. zipper.}! !

!TreeZipper methodsFor: 'accessing' stamp: 'fbs 2/16/2011 07:45'!
trail
	^ trail.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:34'!
up
	^ self move: #safeUp.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:35'!
upOrElse: aBlock
	^ self move: #safeUp orElse: aBlock.! !

Object subclass: #TreeZipperNode
	instanceVariableNames: 'leftNodes rightNodes path value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!TreeZipperNode commentStamp: 'fbs 1/1/2011 18:01' prior: 0!
I am a recursive structure that describes a path through a tree.

  self path

 refers to the previous position in the path, itself a TreeZipperNode. The first entry in a path has

  self path == nil

Being part of a zipper, I naturally represent a context-with-a-hole - that part of a tree minus the part currently under focus.!

!TreeZipperNode class methodsFor: 'instance creation' stamp: 'fbs 12/31/2010 17:07'!
path: aTreeZipperTrail value: anObject leftNodes: leftCollection rightNodes: rightCollection
	^ self new
		path: aTreeZipperTrail
		value: anObject
		leftNodes: leftCollection
		rightNodes: rightCollection.! !

!TreeZipperNode methodsFor: 'accessing' stamp: 'fbs 12/31/2010 00:19'!
leftNodes
	^ leftNodes! !

!TreeZipperNode methodsFor: 'accessing' stamp: 'fbs 12/31/2010 16:22'!
path
	^ path.! !

!TreeZipperNode methodsFor: 'private' stamp: 'fbs 12/31/2010 17:07'!
path: aTreeZipperTrail value: anObject leftNodes: leftCollection rightNodes: rightCollection
	path := aTreeZipperTrail.
	value := anObject.
	leftNodes := leftCollection.
	rightNodes := rightCollection.! !

!TreeZipperNode methodsFor: 'accessing' stamp: 'fbs 12/31/2010 00:19'!
rightNodes
	^ rightNodes.! !

!TreeZipperNode methodsFor: 'accessing' stamp: 'fbs 12/31/2010 17:06'!
value
	^ value.! !

Object subclass: #ZTree
	instanceVariableNames: 'children value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!ZTree commentStamp: 'fbs 12/30/2010 23:32' prior: 0!
I'm a basic implementation of a tree, only present so that my package doesn't need to rely on a separate tree package.!

!ZTree class methodsFor: 'instance creation' stamp: 'fbs 12/30/2010 23:40'!
value: anObject
	^ self new value: anObject children: OrderedCollection new.! !

!ZTree class methodsFor: 'instance creation' stamp: 'fbs 12/30/2010 23:41'!
value: anObject children: aCollection
	^ self new value: anObject children: aCollection.! !

!ZTree methodsFor: 'comparing' stamp: 'fbs 12/31/2010 00:08'!
= anObject
	^ anObject class == self class and:
		[self value = anObject value and:
			[self children = anObject children]].! !

!ZTree methodsFor: 'accessing' stamp: 'fbs 12/30/2010 23:33'!
children
	^ children! !

!ZTree methodsFor: 'initialize-release' stamp: 'fbs 12/30/2010 23:34'!
initialize
	super initialize.
	children := OrderedCollection new.! !

!ZTree methodsFor: 'printing' stamp: 'fbs 12/30/2010 23:54'!
printString
	^ '(', self value printString,
		(children
			ifEmpty: ['']
			ifNotEmpty: [' ', (self children reduce: [:a :b | a printString, ', ', b printString])]),
		')'.! !

!ZTree methodsFor: 'accessing' stamp: 'fbs 12/31/2010 16:54'!
value
	^ value.! !

!ZTree methodsFor: 'private' stamp: 'fbs 12/31/2010 16:54'!
value: anObject children: aCollection
	value := anObject.
	children := aCollection asOrderedCollection.! !

!ZTree methodsFor: 'enumerating' stamp: 'fbs 12/31/2010 19:21'!
zipper
	^ TreeZipper on: self.! !

TestCase subclass: #BinarySearchTreeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!BinarySearchTreeTest methodsFor: 'as yet unclassified' stamp: 'fbs 2/13/2011 17:03'!
testNewTreeIsBlank
	self fail.! !

TestCase subclass: #ListZipperTest
	instanceVariableNames: 'prefix suffix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!ListZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/30/2010 21:13'!
testChangeChangesValue
	| list zipper |
	list := #(1 2 3) asOrderedCollection.
	zipper := ListZipper on: list.
	zipper := zipper right right.
	zipper := zipper change: [:x | x + 1].
	
	zipper := zipper left left.
	
	self assert: zipper focus = #(1 2 4) asOrderedCollection.! !

!ListZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/30/2010 21:13'!
testChangePreservesOriginal
	| original list zipper |
	original := list := #(1 (2 3) 4) asOrderedCollection.
	zipper := ListZipper on: list.
	zipper := zipper right.
	zipper := zipper change: [:x | x size].
	zipper := zipper left.
	
	self assert: zipper focus = #(1 2 4) asOrderedCollection.
	self assert: list = original description: 'Zipper not functional'.! !

!ListZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/30/2010 21:15'!
testChangeSharesStructure
	| original list zipper |
	original := list := #(1 (2 3) 4) asOrderedCollection.
	zipper := ListZipper on: list.
	zipper := zipper right right.
	zipper := zipper change: [:x | x + 1].
	zipper := zipper left left.
	
	self assert: zipper focus = #(1 (2 3) 5) asOrderedCollection.
	self assert: zipper focus second == original second description: 'Zipper not sharing original structure'.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/28/2010 16:50'!
testLeft
	| left list zipper right |
	list := #(1 2 3) asOrderedCollection.
	zipper := ListZipper on: list.
	right := zipper right right.
	
	left := right left.
	self assert: left focus = list allButFirst.
	
	left := left left.
	self assert: left focus = list.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:17'!
testLeftPastBeginningOfCollectionSignalsException
	| zipper |
	zipper := ListZipper on: #(1 2).
	self
		should: [ zipper right left left ]
		raise: ZipperNavigation.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/28/2010 16:49'!
testOn
	| list zipper |
	list := #(1 2 3 4 5).
	zipper := ListZipper on: list.
	self assert: zipper focus = list asOrderedCollection.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/28/2010 16:50'!
testRight
	| list zipper right |
	list := #(1 2 3) asOrderedCollection.
	zipper := ListZipper on: list.
	right := zipper right.
	self assert: right focus = list allButFirst.
	
	right := right right.
	self assert: right focus = list allButFirst allButFirst.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:17'!
testRightPastEndOfCollectionSignalsException
	| zipper |
	zipper := ListZipper on: #(1 2).
	self
		should: [ zipper right right right ]
		raise: ZipperNavigation.! !

TestCase subclass: #PreOrderTraversalTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!PreOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/16/2011 07:32'!
testEmptyTraversalDoesNothing
	| i |
	i := 0.
	PreOrderTraversal new traverse: (ZTree value: 1) doing: [:each | i := i + 1].
	
	self assert: i = 1.! !

!PreOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/16/2011 17:57'!
testTraverseBranchingTreeCallsBlock
	| i |
	i := 0.
	PreOrderTraversal new traverse: (ZTree value: 0 children: {ZTree value: 1. ZTree value: 2}) doing: [:each | i := i + 1].
	
	self assert: i = 3.! !

!PreOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/14/2011 22:26'!
testTreeTraversalCallsBlock
	| i |
	i := 0.
	PreOrderTraversal new traverse: (ZTree value: 1) doing: [:each | i := i + 1].
	
	self assert: i = 1.! !

!PreOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/17/2011 07:51'!
testTreeTraversalMovesDepthFirst
	| i tree |
	tree := ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5}.
	i := #() asOrderedCollection.
	PreOrderTraversal new traverse: tree doing: [:each | i add: each ].
	
	self assert: i = #(1 2 3 4 5) asOrderedCollection.! !

TestCase subclass: #TreeZipperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 12/30/2010 23:39'!
emptyTree
	^ ZTree value: 0.! !

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 12/31/2010 18:21'!
flatTree
	" Best viewed in a monospaced font:
	       1
	   / / | \ \
	  2 3  4 5 6"
	^ ZTree value: 1 children: ((2 to: 6) collect: [ :i | ZTree value: i])! !

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 12/31/2010 17:58'!
nestedTriangleTree
	" Best viewed in a monospaced font:
	    1
	   / \
	  2   3 
	     / \
	    4   5 "
	| b c |
	b := ZTree value: 2.
	c := ZTree value: 3 children: {ZTree value: 4. ZTree value: 5}.
	^ ZTree value: 1 children: {b. c.}! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/31/2010 18:47'!
testChangeReplacesValue
	| tree zipper |
	tree := self triangleTree.
	zipper := (TreeZipper on: tree) down.
	
	zipper := zipper change: [ :i | i + 1].
	self assert: zipper focus value = 3 description: 'No change made'.
	self assert: zipper right focus value = 3 description: 'Position changed'.
	self assert: zipper up focus value = 1 description: 'Position changed'.
	! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/31/2010 19:31'!
testChangeToReplacesValue
	| tree zipper |
	tree := self triangleTree.
	zipper := (TreeZipper on: tree) down.
	
	zipper := zipper changeTo: 5.
	self assert: zipper focus value = 5 description: 'No change made'.
	self assert: zipper right focus value = 3 description: 'Position changed'.
	self assert: zipper up focus value = 1 description: 'Position changed'.
	! !

!TreeZipperTest methodsFor: 'testing - copying' stamp: 'fbs 2/16/2011 07:45'!
testCopyReturnsExactCopy
	| copy zipper |
	zipper := self triangleTree zipper down.
	copy := zipper copy.
	
	self deny: copy == zipper.
	self assert: copy focus = zipper focus.
	self assert: copy up focus = zipper up focus.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 15:55'!
testDeleteAtTopFails
	self should: [self emptyTree zipper delete] raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 16:06'!
testDeleteLeafNode
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}.
	zipper := tree zipper down delete.
	
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 3}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 16:07'!
testDeleteLeafNodeDeletesNode
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}.
	zipper := tree zipper down delete.
	
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 3}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 16:15'!
testDeleteLeafNodeFocusesOnParent
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2}.
	zipper := tree zipper down.
	zipper := zipper delete.
	
	self assert: zipper root = (ZTree value: 1).
	self assert: zipper focus value = 1.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 16:06'!
testDeleteNodeFocusesOnYoungerSibling
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}.
	zipper := tree zipper down delete.
	
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 3}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 17:15'!
testDeleteOldestSiblingFocusesOnYoungerSibling
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3. ZTree value: 4}.
	zipper := tree zipper down.
	self assert: zipper focus value = 2.
	zipper := zipper delete.
	
	self assert: zipper focus value = 3 description: 'Zipper focused on wrong sibling'.
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 3. ZTree value: 4}) description: 'Wrong thing deleted'.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 17:15'!
testDeleteYoungerSiblingFocusesOnOlderSibling
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3. ZTree value: 4}.
	zipper := tree zipper down right.
	self assert: zipper focus value = 3.
	zipper := zipper delete.
	
	self assert: zipper focus value = 2 description: 'Zipper focused on wrong sibling'.
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 2. ZTree value: 4}) description: 'Wrong thing deleted'.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 17:16'!
testDeleteYoungestSiblingFocusesOnOlderSibling
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3. ZTree value: 4}.
	zipper := tree zipper down right right.
	self assert: zipper focus value = 4.
	zipper := zipper delete.
	
	self assert: zipper focus value = 3 description: 'Zipper focused on wrong sibling'.
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}) description: 'Wrong thing deleted'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:18'!
testDownOnEmptyTreeSignalsException
	| zipper |
	zipper := TreeZipper on: self emptyTree.
	
	self
		should: [zipper down]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:59'!
testDownOnLeafNodeSignalsException
	| zipper |
	zipper := TreeZipper on: self triangleTree.
	zipper := zipper down.

	self
		should: [zipper down]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 23:44'!
testDownOnNonEmptyTreeFocusesOnFirstSibling
	| tree zipper |
	tree := self triangleTree.
	zipper := TreeZipper on: tree.
	zipper := zipper down.
	
	self assert: zipper focus = tree children first.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 17:42'!
testDownOrElseCanUseFocus
	| node tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	zipper downOrElse: [:focus | node := focus ].
	
	self assert: node = zipper focus.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 07:50'!
testDownOrElseOnEmptyTreeRunsBlock
	| ran zipper |
	zipper := self emptyTree zipper.
	ran := false.
	zipper := zipper downOrElse: [ran := true].
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:18'!
testDownOrElseOnLeafNodeRunsBlock
	| ran zipper |
	zipper := TreeZipper on: self triangleTree.
	zipper := zipper down.

	ran := false.
	zipper := zipper downOrElse: [ran := true].
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 18:29'!
testDownOrElseOnNonEmptyTreeFocusesOnFirstSibling
	| ran tree zipper |
	tree := self triangleTree.
	zipper := TreeZipper on: tree.
	ran := false.
	zipper := zipper downOrElse: [ran := true].
	
	self assert: zipper focus = tree children first.
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/31/2010 23:46'!
testInsertDownInsertsChild
	| tree zipper |
	tree := ZTree value: 0.
	zipper := tree zipper.
	
	zipper := zipper insertDown: 1.
	"self should: [zipper up] raise: ZipperNavigation."
	self assert: zipper focus value = 1 description: 'Zipper not focused on new node'.
	self shouldnt: [zipper up] raise: ZipperNavigation.
	
	self assert: zipper root = (ZTree value: 0 children: {ZTree value: 1}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 13:47'!
testInsertLeftInsertsImmediatelyOlderSibling
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1. ZTree value: 2}.
	zipper := tree zipper.
	zipper := zipper down right insertLeft: 3.
	self assert: zipper root = (ZTree value: 0 children: {ZTree value: 1. ZTree value: 3. ZTree value: 2}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 13:20'!
testInsertLeftInsertsOlderSibling
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1}.
	zipper := tree zipper.
	zipper := zipper down insertLeft: 2.
	self assert: zipper root = (ZTree value: 0 children: {ZTree value: 2. ZTree value: 1}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/31/2010 23:52'!
testInsertRightInsertsYoungerSibling
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1}.
	zipper := tree zipper.
	zipper := zipper down insertRight: 2.
	self assert: zipper root = (ZTree value: 0 children: {ZTree value: 1. ZTree value: 2}).! !

!TreeZipperTest methodsFor: 'testing - testing' stamp: 'fbs 2/17/2011 07:38'!
testIsEmptyOnEmptyTreeReturnsTrue
	self assert: self emptyTree zipper isEmpty.! !

!TreeZipperTest methodsFor: 'testing - testing' stamp: 'fbs 2/17/2011 07:42'!
testIsEmptyOnNonEmptyTreeAlwaysReturnsFalse
	| zipper |
	zipper := self triangleTree zipper.
	self deny: zipper isEmpty.
	
	self deny: zipper down isEmpty.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 18:48'!
testLeftAtLeftmostNodeSignalsException
	| zipper |
	zipper := (TreeZipper on: self triangleTree) down.
	
	self
		should: [zipper left]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 17:29'!
testLeftAtRootNodeSignalsException
	| zipper |
	zipper := TreeZipper on: self triangleTree.
	
	self
		should: [zipper left]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 18:22'!
testLeftMovesToLeftSibling
	| tree zipper |
	tree := self nestedTriangleTree.
	zipper := TreeZipper on: tree.
	zipper := zipper down right down up.
	
	self assert: zipper focus value = 3 description: 'We followed the wrong path'.

	zipper := zipper left.
	self assert: zipper focus value = 2 description: 'Left didn''t go left'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:17'!
testLeftOrElseAtLeftmostNodeRunsBlock
	| ran zipper |
	zipper := (TreeZipper on: self triangleTree) down.
	
	zipper := zipper leftOrElse: [ran := true].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:17'!
testLeftOrElseAtRootNodeRunsBlock
	| ran zipper |
	zipper := TreeZipper on: self triangleTree.
	
	zipper := zipper leftOrElse: [ran := true].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 17:41'!
testLeftOrElseCanUseFocus
	| node tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	zipper leftOrElse: [:focus | node := focus ].
	
	self assert: node = zipper focus.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 18:34'!
testLeftOrElseMovesToLeftSibling
	| ran tree zipper |
	tree := self nestedTriangleTree.
	zipper := TreeZipper on: tree.
	zipper := zipper down right down up.
	
	self assert: zipper focus value = 3 description: 'We followed the wrong path'.

	ran := false.
	zipper := zipper leftOrElse: [ran := true].
	self assert: zipper focus value = 2 description: 'Left didn''t go left'.
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:34'!
testLeftRightIdempotent
	| tree zipper |
	tree := self flatTree.
	zipper := tree zipper down.
	4 timesRepeat: [zipper := zipper right].
	self assert: zipper focus value = 6.
	
	"Node 2 has 4 younger siblings, so 4 is as far as we can navigate with #right.."
	1 to: 4 do:
		[ :i |
			| original navigated |
			original := navigated :=zipper.
			i timesRepeat: [ navigated := navigated left ].
			i timesRepeat: [ navigated := navigated right ].
			self assert: original focus = navigated focus ]! !

!TreeZipperTest methodsFor: 'testing - instance creation' stamp: 'fbs 12/30/2010 22:19'!
testOn
	| tree zipper |
	tree := self emptyTree.
	zipper := TreeZipper on: tree.
	
	self assert: zipper focus = tree.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 18:49'!
testRightAtRightmostNodeSignalsException
	| zipper |
	zipper := (TreeZipper on: self triangleTree) down right.
	
	self
		should: [zipper right]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 18:42'!
testRightAtRootNodeSignalsException
	| zipper |
	zipper := TreeZipper on: self triangleTree.
	
	self
		should: [zipper right]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:35'!
testRightLeftIdempotent
	| tree zipper |
	tree := self flatTree.
	zipper := tree zipper down.
	
	"Node 2 has 4 younger siblings, so 4 is as far as we can navigate with #right.."
	1 to: 4 do:
		[ :i |
			| original navigated |
			original := navigated :=zipper.
			i timesRepeat: [ navigated := navigated right ].
			i timesRepeat: [ navigated := navigated left ].
			self assert: original focus = navigated focus ]! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:27'!
testRightMovesToRightSibling
	| tree zipper |
	tree := self triangleTree.
	zipper := tree zipper.
	zipper := zipper down right.
	
	self assert: zipper focus = (tree children at: 2).! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:15'!
testRightOrElseAtRightmostNodeRunsBlock
	| ran zipper |
	zipper := (TreeZipper on: self triangleTree) down right.
	
	ran := false.
	zipper := zipper rightOrElse: [ran := true].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:15'!
testRightOrElseAtRootNodeRunsBlock
	| ran zipper |
	zipper := TreeZipper on: self triangleTree.
	
	zipper := zipper rightOrElse: [ran := true].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 17:41'!
testRightOrElseCanUseFocus
	| node tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	zipper rightOrElse: [:focus | node := focus ].
	
	self assert: node = zipper focus.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 18:41'!
testRightOrElseMovesToRightSibling
	| ran tree zipper |
	tree := self triangleTree.
	zipper := tree zipper.
	zipper := zipper down.
	
	ran := false.
	zipper := zipper rightOrElse: [ran := true].
	
	self assert: zipper focus = (tree children at: 2).
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:48'!
testRightStoresValueCorrectly
	"#right didn't store the value correctly, and would cause the right-most node
	value to be written to all the subsequently-visited nodes."
	| tree zipper |
	tree := self nestedTriangleTree.
	zipper := tree zipper.
	"Down, any (valid) combination of right/left, then up: no-op."
	self assert: zipper focus value = zipper down right up focus value.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 23:31'!
testRootAtChildNodeReturnsRoot
	| tree zipper |
	tree := self triangleTree.
	zipper := tree zipper down.
	
	self assert: zipper root = tree.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 23:29'!
testRootAtRootReturnsRoot
	| tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	
	self assert: zipper root == tree.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:26'!
testUpAtRootNodeSignalsException
	| zipper |
	zipper := self triangleTree zipper.
	
	self
		should: [zipper up]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:14'!
testUpOrElseAtRootNodeRunsBlock
	| ran zipper |
	zipper := self triangleTree zipper.
	
	ran := false.
	zipper := zipper upOrElse: [ran := true.].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 17:41'!
testUpOrElseCanUseFocus
	| node tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	zipper rightOrElse: [:focus | node := focus ].
	
	self assert: node = zipper focus.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:20'!
testUpOrElseReturnsToParentNode
	| ran tree zipper |
	tree := self triangleTree.
	zipper := tree zipper.
	ran := false.
	zipper := zipper down upOrElse: [ran := true].
	
	self assert: zipper focus = tree.
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:21'!
testUpOrElseReturnsToParentNode2
	| ran tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1 children: {ZTree value: 2}}.
	zipper := tree zipper.
	ran := false.
	zipper := zipper down down up upOrElse: [ran := true].
	
	self assert: zipper focus = tree.
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:26'!
testUpReturnsToParentNode
	| tree zipper |
	tree := self triangleTree.
	zipper := tree zipper.
	zipper := zipper down up.
	
	self assert: zipper focus = tree.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 14:46'!
testUpReturnsToParentNode2
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1 children: {ZTree value: 2}}.
	zipper := tree zipper.
	zipper := zipper down down up up.
	
	self assert: zipper focus = tree.! !

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 12/31/2010 16:53'!
triangleTree
	" Best viewed in a monospaced font:
	   1
	  / \
	 2   3 "
	| b c |
	b := ZTree value: 2.
	c := ZTree value: 3.
	^ ZTree value: 1 children: {b. c.}! !

TestCase subclass: #ZTreeTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:35'!
testEqualsEqualSingleNodesReturnsTrue
	| a b |
	a := ZTree value: 1.
	b := ZTree value: 1.
	self assert: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:36'!
testEqualsEqualTreesReturnsTrue
	| a b |
	a := ZTree value: 1 children: {ZTree value: 2}.
	b := ZTree value: 1 children: {ZTree value: 2}.
	self assert: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:35'!
testEqualsUnequalSingleNodesReturnsFalse
	| a b |
	a := ZTree value: 1.
	b := ZTree value: 2.
	self deny: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:37'!
testEqualsUnequalTreesInChildrenReturnsFalse
	| a b |
	a := ZTree value: 1 children: {ZTree value: 2}.
	b := ZTree value: 1 children: {ZTree value: 1}.
	self deny: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:39'!
testEqualsUnequalTreesInValueReturnsFalse
	| a b |
	a := ZTree value: 1 children: {ZTree value: 2}.
	b := ZTree value: 2 children: {ZTree value: 2}.
	self deny: a = b.! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 12/31/2010 23:32'!
testPrintSingleNodePrintsNoChildren
	self assert: (ZTree value: 1) printString = '(1)'.! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 12/31/2010 23:33'!
testPrintTreeNodePrintsChildren
	| t |
	t := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}.
	self assert: t printString = '(1 (2), (3))'.! !

Error subclass: #ZipperNavigation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!ZipperNavigation commentStamp: 'fbs 12/28/2010 17:36' prior: 0!
Raise me whenever a Zipper tries to navigate to somewhere it cannot - past the end of a list, to a missing subtree, etc.!
